from typing import Any
import numpy as np
import cmath

from ft4ftir.interferogram import Interferogram
from ft4ftir.apodization import Apodization

class FourierTransform:

    def __init__(
        self,
        interferogram: Interferogram,
        apodization: Apodization
    ) -> None:
        
        self.interferogram = interferogram
        self.apodization = apodization

    def get_next_power_of_two(self, n):
        """
        Calculate the next power of two greater than or equal to n.

        Parameters:
        - n (int): Input integer.

        Returns:
        - int: The next power of two.
        """
        return int(2 ** np.ceil(np.log2(n)))
    
    def zero_fill(self, interferogram, target_size):
        """
        Zero-fill the interferogram to a target size.

        Parameters:
        - interferogram (numpy.ndarray): Input interferogram data.
        - target_size (int): Target size after zero-filling.

        Returns:
        - numpy.ndarray: Zero-filled interferogram.
        """
        filled_interferogram = np.zeros(target_size)
        half_size = int(interferogram.size / 2)
        
        filled_interferogram[:half_size] = interferogram[half_size:]
        filled_interferogram[-half_size:] = interferogram[:half_size][::-1]
        
        return filled_interferogram

    def __call__(self, *args: Any, **kwds: Any) -> Any:
        
        # Preprocess the interferogram
        apodized_interferogram = self.apodization(self.interferogram.interferogram)
        next_power_of_two = self.get_next_power_of_two(apodized_interferogram.size)
        reversed_zerofilled_interferogram = self.zero_fill(apodized_interferogram, target_size=next_power_of_two) 

        # FFT
        uncorrected_spectrum = np.fft.fft(reversed_zerofilled_interferogram)

        # Phase correction
        corrected_spectrum_size = int(next_power_of_two / 2)
        corrected_spectrum = np.zeros(corrected_spectrum_size)

        for i in range(int(corrected_spectrum_size / 2)):
            # Calculate phase
            phase = cmath.phase(uncorrected_spectrum[i])
            
            # Apply phase correction
            corrected_spectrum[i] = np.abs(np.real(uncorrected_spectrum[i] * np.exp(-1j * phase)))
            
        # Generate wavenumbers
        wavenumbers = np.zeros(corrected_spectrum_size)
        wavenumbers = np.arange(corrected_spectrum_size) * self.interferogram.laser_wavenumber / corrected_spectrum_size
        
        return corrected_spectrum, wavenumbers



from typing import Any, Optional
import numpy as np

class Apodization:

    def __init__(
        self, 
        name: str, 
        size: int,
        window: Optional[np.array] = np.array([])
    ) -> None:
        
        self.apodization_functions = {
            "BlackmanHarris3Term": self.blackman_harris_3_term,
        }

        self.name = name 
        self.size = size 
        self.window = window 
        self.function = None

        if self.window.size == 0:

            self.function = self.apodization_functions.get(self.name)
            self.window = self.function(size = self.size)

    def __call__(self, interferogram) -> Any:
        return interferogram*self.window
    
    def blackman_harris_3_term(self, size: Optional[int] = None):
        if size is None:
            size = self.size
        return np.blackman(size)







from typing import Any


class FourierTransform:

    def __init__(self, param1:int, param2: float, **kwargs) -> None:
        pass

    def getter(self):
        pass

    def __call__(self, interferogram, *args: Any, **kwds: Any) -> Any:
        a = self.apodization * interferogram

        return a
    

if __name__ == "__main__":

    ft = FourierTransform()
    array1 = []
    array2 = []

    result = ft(array1, array2)
    print(result)


from setuptools import setup, find_packages

setup(
    name="ft4ftirs",
    version="0.1.0",
    author="Jon Gabirondo-López",
    author_email="jon.gabirondol@ehu.eus",
    description="A package for processing FTIR interferograms.",
    long_description=open('README.md').read(),
    long_description_content_type="text/markdown",
    url="https://github.com/jongablop/ft4ftirs",
    packages=find_packages(),
    include_package_data=True,
    install_requires=[
        "numpy>=1.21.0",
        "pandas>=1.3.0",
        "brukeropusreader>=1.3.0"
    ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.7',
)



import numpy as np
from typing import Union

class Interferogram:

    def __init__(
        self,
        x_index: np.array,
        interferogram: np.array,
        laser_wavenumber: Union[int, float]    
    ) -> None:
        
        self.laser_wavenumber = laser_wavenumber
        self.x_index = x_index
        self.interferogram = interferogram


from brukeropusreader import read_file
import numpy as np

def load_opus_file(opus_file_path):
    """
    Load an OPUS file and extract the relevant information.

    Parameters:
    - opus_file_path (str): Path to the OPUS file.

    Returns:
    - dict: A dictionary containing the interferogram, Fourier transform parameters, and signal parameters.
    """
    opus_file = read_file(opus_file_path)
    
    # Extract signal gain first (required in the interferogram processing)
    signal_gain = opus_file.get('Instrument').get('ASG')

    # Interferogram parameters
    interferogram_scaling_factor = opus_file.get('IgSm Data Parameter').get('CSF')
    laser_wavenumber = opus_file.get('Instrument').get('HFL')
    original_interferogram = opus_file['IgSm']
    x_index = opus_file.get_range('IgSm')
    
    scaled_interferogram = original_interferogram * interferogram_scaling_factor
    
    interferogram_parameters = {
        'x_index': np.array(x_index),
        'interferogram': np.array(scaled_interferogram),
        'laser_wavenumber': laser_wavenumber,
    }

    # Fourier transform parameters
    apodization_functions_naming = {
        'B3': 'BlackmanHarris3Term'
        # Add other mappings as needed
    }

    apodization_function = opus_file.get('Fourier Transformation').get('APF')

    ft_parameters = {
        'apodization_function_name': apodization_functions_naming.get(apodization_function, 'Unknown')
    }

    # Signal parameters
    signal_parameters = {
        'signal_gain': signal_gain
    }

    return {
        'interferogram': interferogram_parameters,
        'fourier_transform': ft_parameters,
        'signal': signal_parameters
    }
